#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 4
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGER2K(ATL_CINT M, ATL_CINT N, const float *X, const float *Y, 
                const float *W, const float *Z, float *A, ATL_CINT lda)
/*
 * 112x4 unrolled r2_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N4=(N/4)*4, M112=(M/112)*112, lda4=lda*4;
   ATL_INT j;

   for (j=N4; j; j -= 4, A += lda4, Y += 4)
   {
      float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda;
      const register float y0=Y[0], z0=Z[0], y1=Y[1], z1=Z[1], y2=Y[2], z2=Z[2],
                           y3=Y[3], z3=Z[3];
      ATL_INT i;
      prefY(Y+4+4-1);
      prefY(Z+4+4-1);
      for (i=0; i < M112; i += 112)
      {
         register float a0, a1, a2, a3;
         const float x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                     w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                     x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                     w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                     x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                     x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                     x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15],
                     x16=X[i+16], w16=W[i+16], x17=X[i+17], w17=W[i+17],
                     x18=X[i+18], w18=W[i+18], x19=X[i+19], w19=W[i+19],
                     x20=X[i+20], w20=W[i+20], x21=X[i+21], w21=W[i+21],
                     x22=X[i+22], w22=W[i+22], x23=X[i+23], w23=W[i+23],
                     x24=X[i+24], w24=W[i+24], x25=X[i+25], w25=W[i+25],
                     x26=X[i+26], w26=W[i+26], x27=X[i+27], w27=W[i+27],
                     x28=X[i+28], w28=W[i+28], x29=X[i+29], w29=W[i+29],
                     x30=X[i+30], w30=W[i+30], x31=X[i+31], w31=W[i+31],
                     x32=X[i+32], w32=W[i+32], x33=X[i+33], w33=W[i+33],
                     x34=X[i+34], w34=W[i+34], x35=X[i+35], w35=W[i+35],
                     x36=X[i+36], w36=W[i+36], x37=X[i+37], w37=W[i+37],
                     x38=X[i+38], w38=W[i+38], x39=X[i+39], w39=W[i+39],
                     x40=X[i+40], w40=W[i+40], x41=X[i+41], w41=W[i+41],
                     x42=X[i+42], w42=W[i+42], x43=X[i+43], w43=W[i+43],
                     x44=X[i+44], w44=W[i+44], x45=X[i+45], w45=W[i+45],
                     x46=X[i+46], w46=W[i+46], x47=X[i+47], w47=W[i+47],
                     x48=X[i+48], w48=W[i+48], x49=X[i+49], w49=W[i+49],
                     x50=X[i+50], w50=W[i+50], x51=X[i+51], w51=W[i+51],
                     x52=X[i+52], w52=W[i+52], x53=X[i+53], w53=W[i+53],
                     x54=X[i+54], w54=W[i+54], x55=X[i+55], w55=W[i+55],
                     x56=X[i+56], w56=W[i+56], x57=X[i+57], w57=W[i+57],
                     x58=X[i+58], w58=W[i+58], x59=X[i+59], w59=W[i+59],
                     x60=X[i+60], w60=W[i+60], x61=X[i+61], w61=W[i+61],
                     x62=X[i+62], w62=W[i+62], x63=X[i+63], w63=W[i+63],
                     x64=X[i+64], w64=W[i+64], x65=X[i+65], w65=W[i+65],
                     x66=X[i+66], w66=W[i+66], x67=X[i+67], w67=W[i+67],
                     x68=X[i+68], w68=W[i+68], x69=X[i+69], w69=W[i+69],
                     x70=X[i+70], w70=W[i+70], x71=X[i+71], w71=W[i+71],
                     x72=X[i+72], w72=W[i+72], x73=X[i+73], w73=W[i+73],
                     x74=X[i+74], w74=W[i+74], x75=X[i+75], w75=W[i+75],
                     x76=X[i+76], w76=W[i+76], x77=X[i+77], w77=W[i+77],
                     x78=X[i+78], w78=W[i+78], x79=X[i+79], w79=W[i+79],
                     x80=X[i+80], w80=W[i+80], x81=X[i+81], w81=W[i+81],
                     x82=X[i+82], w82=W[i+82], x83=X[i+83], w83=W[i+83],
                     x84=X[i+84], w84=W[i+84], x85=X[i+85], w85=W[i+85],
                     x86=X[i+86], w86=W[i+86], x87=X[i+87], w87=W[i+87],
                     x88=X[i+88], w88=W[i+88], x89=X[i+89], w89=W[i+89],
                     x90=X[i+90], w90=W[i+90], x91=X[i+91], w91=W[i+91],
                     x92=X[i+92], w92=W[i+92], x93=X[i+93], w93=W[i+93],
                     x94=X[i+94], w94=W[i+94], x95=X[i+95], w95=W[i+95],
                     x96=X[i+96], w96=W[i+96], x97=X[i+97], w97=W[i+97],
                     x98=X[i+98], w98=W[i+98], x99=X[i+99], w99=W[i+99],
                     x100=X[i+100], w100=W[i+100], x101=X[i+101], w101=W[i+101],
                     x102=X[i+102], w102=W[i+102], x103=X[i+103], w103=W[i+103],
                     x104=X[i+104], w104=W[i+104], x105=X[i+105], w105=W[i+105],
                     x106=X[i+106], w106=W[i+106], x107=X[i+107], w107=W[i+107],
                     x108=X[i+108], w108=W[i+108], x109=X[i+109], w109=W[i+109],
                     x110=X[i+110], w110=W[i+110], x111=X[i+111],
                     w111=W[i+111];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         a1 = A1[i+0];
         a1 += x0 * y1;
         a2 = A2[i+0];
         a2 += x0 * y2;
         a3 = A3[i+0];
         a3 += x0 * y3;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a1 += w0 * z1;
         prefA(A1);
         A1[i+0] = a1;
         a2 += w0 * z2;
         prefA(A2);
         A2[i+0] = a2;
         a3 += w0 * z3;
         prefA(A3);
         A3[i+0] = a3;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a1 = A1[i+1];
         a1 += x1 * y1;
         a2 = A2[i+1];
         a2 += x1 * y2;
         a3 = A3[i+1];
         a3 += x1 * y3;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a1 += w1 * z1;
         A1[i+1] = a1;
         a2 += w1 * z2;
         A2[i+1] = a2;
         a3 += w1 * z3;
         A3[i+1] = a3;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a1 = A1[i+2];
         a1 += x2 * y1;
         a2 = A2[i+2];
         a2 += x2 * y2;
         a3 = A3[i+2];
         a3 += x2 * y3;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a1 += w2 * z1;
         A1[i+2] = a1;
         a2 += w2 * z2;
         A2[i+2] = a2;
         a3 += w2 * z3;
         A3[i+2] = a3;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a1 = A1[i+3];
         a1 += x3 * y1;
         a2 = A2[i+3];
         a2 += x3 * y2;
         a3 = A3[i+3];
         a3 += x3 * y3;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a1 += w3 * z1;
         A1[i+3] = a1;
         a2 += w3 * z2;
         A2[i+3] = a2;
         a3 += w3 * z3;
         A3[i+3] = a3;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a1 = A1[i+4];
         a1 += x4 * y1;
         a2 = A2[i+4];
         a2 += x4 * y2;
         a3 = A3[i+4];
         a3 += x4 * y3;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a1 += w4 * z1;
         A1[i+4] = a1;
         a2 += w4 * z2;
         A2[i+4] = a2;
         a3 += w4 * z3;
         A3[i+4] = a3;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a1 = A1[i+5];
         a1 += x5 * y1;
         a2 = A2[i+5];
         a2 += x5 * y2;
         a3 = A3[i+5];
         a3 += x5 * y3;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a1 += w5 * z1;
         A1[i+5] = a1;
         a2 += w5 * z2;
         A2[i+5] = a2;
         a3 += w5 * z3;
         A3[i+5] = a3;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a1 = A1[i+6];
         a1 += x6 * y1;
         a2 = A2[i+6];
         a2 += x6 * y2;
         a3 = A3[i+6];
         a3 += x6 * y3;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a1 += w6 * z1;
         A1[i+6] = a1;
         a2 += w6 * z2;
         A2[i+6] = a2;
         a3 += w6 * z3;
         A3[i+6] = a3;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a1 = A1[i+7];
         a1 += x7 * y1;
         a2 = A2[i+7];
         a2 += x7 * y2;
         a3 = A3[i+7];
         a3 += x7 * y3;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a1 += w7 * z1;
         A1[i+7] = a1;
         a2 += w7 * z2;
         A2[i+7] = a2;
         a3 += w7 * z3;
         A3[i+7] = a3;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a1 = A1[i+8];
         a1 += x8 * y1;
         a2 = A2[i+8];
         a2 += x8 * y2;
         a3 = A3[i+8];
         a3 += x8 * y3;
         a0 += w8 * z0;
         A0[i+8] = a0;
         a1 += w8 * z1;
         A1[i+8] = a1;
         a2 += w8 * z2;
         A2[i+8] = a2;
         a3 += w8 * z3;
         A3[i+8] = a3;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a1 = A1[i+9];
         a1 += x9 * y1;
         a2 = A2[i+9];
         a2 += x9 * y2;
         a3 = A3[i+9];
         a3 += x9 * y3;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a1 += w9 * z1;
         A1[i+9] = a1;
         a2 += w9 * z2;
         A2[i+9] = a2;
         a3 += w9 * z3;
         A3[i+9] = a3;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a1 = A1[i+10];
         a1 += x10 * y1;
         a2 = A2[i+10];
         a2 += x10 * y2;
         a3 = A3[i+10];
         a3 += x10 * y3;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a1 += w10 * z1;
         A1[i+10] = a1;
         a2 += w10 * z2;
         A2[i+10] = a2;
         a3 += w10 * z3;
         A3[i+10] = a3;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a1 = A1[i+11];
         a1 += x11 * y1;
         a2 = A2[i+11];
         a2 += x11 * y2;
         a3 = A3[i+11];
         a3 += x11 * y3;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a1 += w11 * z1;
         A1[i+11] = a1;
         a2 += w11 * z2;
         A2[i+11] = a2;
         a3 += w11 * z3;
         A3[i+11] = a3;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a1 = A1[i+12];
         a1 += x12 * y1;
         a2 = A2[i+12];
         a2 += x12 * y2;
         a3 = A3[i+12];
         a3 += x12 * y3;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a1 += w12 * z1;
         A1[i+12] = a1;
         a2 += w12 * z2;
         A2[i+12] = a2;
         a3 += w12 * z3;
         A3[i+12] = a3;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a1 = A1[i+13];
         a1 += x13 * y1;
         a2 = A2[i+13];
         a2 += x13 * y2;
         a3 = A3[i+13];
         a3 += x13 * y3;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a1 += w13 * z1;
         A1[i+13] = a1;
         a2 += w13 * z2;
         A2[i+13] = a2;
         a3 += w13 * z3;
         A3[i+13] = a3;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a1 = A1[i+14];
         a1 += x14 * y1;
         a2 = A2[i+14];
         a2 += x14 * y2;
         a3 = A3[i+14];
         a3 += x14 * y3;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a1 += w14 * z1;
         A1[i+14] = a1;
         a2 += w14 * z2;
         A2[i+14] = a2;
         a3 += w14 * z3;
         A3[i+14] = a3;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a1 = A1[i+15];
         a1 += x15 * y1;
         a2 = A2[i+15];
         a2 += x15 * y2;
         a3 = A3[i+15];
         a3 += x15 * y3;
         a0 += w15 * z0;
         A0[i+15] = a0;
         a1 += w15 * z1;
         A1[i+15] = a1;
         a2 += w15 * z2;
         A2[i+15] = a2;
         a3 += w15 * z3;
         A3[i+15] = a3;
         a0 = A0[i+16];
         a0 += x16 * y0;
         a1 = A1[i+16];
         a1 += x16 * y1;
         a2 = A2[i+16];
         a2 += x16 * y2;
         a3 = A3[i+16];
         a3 += x16 * y3;
         a0 += w16 * z0;
         prefA(A0);
         A0[i+16] = a0;
         a1 += w16 * z1;
         prefA(A1);
         A1[i+16] = a1;
         a2 += w16 * z2;
         prefA(A2);
         A2[i+16] = a2;
         a3 += w16 * z3;
         prefA(A3);
         A3[i+16] = a3;
         a0 = A0[i+17];
         a0 += x17 * y0;
         a1 = A1[i+17];
         a1 += x17 * y1;
         a2 = A2[i+17];
         a2 += x17 * y2;
         a3 = A3[i+17];
         a3 += x17 * y3;
         a0 += w17 * z0;
         A0[i+17] = a0;
         a1 += w17 * z1;
         A1[i+17] = a1;
         a2 += w17 * z2;
         A2[i+17] = a2;
         a3 += w17 * z3;
         A3[i+17] = a3;
         a0 = A0[i+18];
         a0 += x18 * y0;
         a1 = A1[i+18];
         a1 += x18 * y1;
         a2 = A2[i+18];
         a2 += x18 * y2;
         a3 = A3[i+18];
         a3 += x18 * y3;
         a0 += w18 * z0;
         A0[i+18] = a0;
         a1 += w18 * z1;
         A1[i+18] = a1;
         a2 += w18 * z2;
         A2[i+18] = a2;
         a3 += w18 * z3;
         A3[i+18] = a3;
         a0 = A0[i+19];
         a0 += x19 * y0;
         a1 = A1[i+19];
         a1 += x19 * y1;
         a2 = A2[i+19];
         a2 += x19 * y2;
         a3 = A3[i+19];
         a3 += x19 * y3;
         a0 += w19 * z0;
         A0[i+19] = a0;
         a1 += w19 * z1;
         A1[i+19] = a1;
         a2 += w19 * z2;
         A2[i+19] = a2;
         a3 += w19 * z3;
         A3[i+19] = a3;
         a0 = A0[i+20];
         a0 += x20 * y0;
         a1 = A1[i+20];
         a1 += x20 * y1;
         a2 = A2[i+20];
         a2 += x20 * y2;
         a3 = A3[i+20];
         a3 += x20 * y3;
         a0 += w20 * z0;
         A0[i+20] = a0;
         a1 += w20 * z1;
         A1[i+20] = a1;
         a2 += w20 * z2;
         A2[i+20] = a2;
         a3 += w20 * z3;
         A3[i+20] = a3;
         a0 = A0[i+21];
         a0 += x21 * y0;
         a1 = A1[i+21];
         a1 += x21 * y1;
         a2 = A2[i+21];
         a2 += x21 * y2;
         a3 = A3[i+21];
         a3 += x21 * y3;
         a0 += w21 * z0;
         A0[i+21] = a0;
         a1 += w21 * z1;
         A1[i+21] = a1;
         a2 += w21 * z2;
         A2[i+21] = a2;
         a3 += w21 * z3;
         A3[i+21] = a3;
         a0 = A0[i+22];
         a0 += x22 * y0;
         a1 = A1[i+22];
         a1 += x22 * y1;
         a2 = A2[i+22];
         a2 += x22 * y2;
         a3 = A3[i+22];
         a3 += x22 * y3;
         a0 += w22 * z0;
         A0[i+22] = a0;
         a1 += w22 * z1;
         A1[i+22] = a1;
         a2 += w22 * z2;
         A2[i+22] = a2;
         a3 += w22 * z3;
         A3[i+22] = a3;
         a0 = A0[i+23];
         a0 += x23 * y0;
         a1 = A1[i+23];
         a1 += x23 * y1;
         a2 = A2[i+23];
         a2 += x23 * y2;
         a3 = A3[i+23];
         a3 += x23 * y3;
         a0 += w23 * z0;
         A0[i+23] = a0;
         a1 += w23 * z1;
         A1[i+23] = a1;
         a2 += w23 * z2;
         A2[i+23] = a2;
         a3 += w23 * z3;
         A3[i+23] = a3;
         a0 = A0[i+24];
         a0 += x24 * y0;
         a1 = A1[i+24];
         a1 += x24 * y1;
         a2 = A2[i+24];
         a2 += x24 * y2;
         a3 = A3[i+24];
         a3 += x24 * y3;
         a0 += w24 * z0;
         A0[i+24] = a0;
         a1 += w24 * z1;
         A1[i+24] = a1;
         a2 += w24 * z2;
         A2[i+24] = a2;
         a3 += w24 * z3;
         A3[i+24] = a3;
         a0 = A0[i+25];
         a0 += x25 * y0;
         a1 = A1[i+25];
         a1 += x25 * y1;
         a2 = A2[i+25];
         a2 += x25 * y2;
         a3 = A3[i+25];
         a3 += x25 * y3;
         a0 += w25 * z0;
         A0[i+25] = a0;
         a1 += w25 * z1;
         A1[i+25] = a1;
         a2 += w25 * z2;
         A2[i+25] = a2;
         a3 += w25 * z3;
         A3[i+25] = a3;
         a0 = A0[i+26];
         a0 += x26 * y0;
         a1 = A1[i+26];
         a1 += x26 * y1;
         a2 = A2[i+26];
         a2 += x26 * y2;
         a3 = A3[i+26];
         a3 += x26 * y3;
         a0 += w26 * z0;
         A0[i+26] = a0;
         a1 += w26 * z1;
         A1[i+26] = a1;
         a2 += w26 * z2;
         A2[i+26] = a2;
         a3 += w26 * z3;
         A3[i+26] = a3;
         a0 = A0[i+27];
         a0 += x27 * y0;
         a1 = A1[i+27];
         a1 += x27 * y1;
         a2 = A2[i+27];
         a2 += x27 * y2;
         a3 = A3[i+27];
         a3 += x27 * y3;
         a0 += w27 * z0;
         A0[i+27] = a0;
         a1 += w27 * z1;
         A1[i+27] = a1;
         a2 += w27 * z2;
         A2[i+27] = a2;
         a3 += w27 * z3;
         A3[i+27] = a3;
         a0 = A0[i+28];
         a0 += x28 * y0;
         a1 = A1[i+28];
         a1 += x28 * y1;
         a2 = A2[i+28];
         a2 += x28 * y2;
         a3 = A3[i+28];
         a3 += x28 * y3;
         a0 += w28 * z0;
         A0[i+28] = a0;
         a1 += w28 * z1;
         A1[i+28] = a1;
         a2 += w28 * z2;
         A2[i+28] = a2;
         a3 += w28 * z3;
         A3[i+28] = a3;
         a0 = A0[i+29];
         a0 += x29 * y0;
         a1 = A1[i+29];
         a1 += x29 * y1;
         a2 = A2[i+29];
         a2 += x29 * y2;
         a3 = A3[i+29];
         a3 += x29 * y3;
         a0 += w29 * z0;
         A0[i+29] = a0;
         a1 += w29 * z1;
         A1[i+29] = a1;
         a2 += w29 * z2;
         A2[i+29] = a2;
         a3 += w29 * z3;
         A3[i+29] = a3;
         a0 = A0[i+30];
         a0 += x30 * y0;
         a1 = A1[i+30];
         a1 += x30 * y1;
         a2 = A2[i+30];
         a2 += x30 * y2;
         a3 = A3[i+30];
         a3 += x30 * y3;
         a0 += w30 * z0;
         A0[i+30] = a0;
         a1 += w30 * z1;
         A1[i+30] = a1;
         a2 += w30 * z2;
         A2[i+30] = a2;
         a3 += w30 * z3;
         A3[i+30] = a3;
         a0 = A0[i+31];
         a0 += x31 * y0;
         a1 = A1[i+31];
         a1 += x31 * y1;
         a2 = A2[i+31];
         a2 += x31 * y2;
         a3 = A3[i+31];
         a3 += x31 * y3;
         a0 += w31 * z0;
         A0[i+31] = a0;
         a1 += w31 * z1;
         A1[i+31] = a1;
         a2 += w31 * z2;
         A2[i+31] = a2;
         a3 += w31 * z3;
         A3[i+31] = a3;
         a0 = A0[i+32];
         a0 += x32 * y0;
         a1 = A1[i+32];
         a1 += x32 * y1;
         a2 = A2[i+32];
         a2 += x32 * y2;
         a3 = A3[i+32];
         a3 += x32 * y3;
         a0 += w32 * z0;
         prefA(A0);
         A0[i+32] = a0;
         a1 += w32 * z1;
         prefA(A1);
         A1[i+32] = a1;
         a2 += w32 * z2;
         prefA(A2);
         A2[i+32] = a2;
         a3 += w32 * z3;
         prefA(A3);
         A3[i+32] = a3;
         a0 = A0[i+33];
         a0 += x33 * y0;
         a1 = A1[i+33];
         a1 += x33 * y1;
         a2 = A2[i+33];
         a2 += x33 * y2;
         a3 = A3[i+33];
         a3 += x33 * y3;
         a0 += w33 * z0;
         A0[i+33] = a0;
         a1 += w33 * z1;
         A1[i+33] = a1;
         a2 += w33 * z2;
         A2[i+33] = a2;
         a3 += w33 * z3;
         A3[i+33] = a3;
         a0 = A0[i+34];
         a0 += x34 * y0;
         a1 = A1[i+34];
         a1 += x34 * y1;
         a2 = A2[i+34];
         a2 += x34 * y2;
         a3 = A3[i+34];
         a3 += x34 * y3;
         a0 += w34 * z0;
         A0[i+34] = a0;
         a1 += w34 * z1;
         A1[i+34] = a1;
         a2 += w34 * z2;
         A2[i+34] = a2;
         a3 += w34 * z3;
         A3[i+34] = a3;
         a0 = A0[i+35];
         a0 += x35 * y0;
         a1 = A1[i+35];
         a1 += x35 * y1;
         a2 = A2[i+35];
         a2 += x35 * y2;
         a3 = A3[i+35];
         a3 += x35 * y3;
         a0 += w35 * z0;
         A0[i+35] = a0;
         a1 += w35 * z1;
         A1[i+35] = a1;
         a2 += w35 * z2;
         A2[i+35] = a2;
         a3 += w35 * z3;
         A3[i+35] = a3;
         a0 = A0[i+36];
         a0 += x36 * y0;
         a1 = A1[i+36];
         a1 += x36 * y1;
         a2 = A2[i+36];
         a2 += x36 * y2;
         a3 = A3[i+36];
         a3 += x36 * y3;
         a0 += w36 * z0;
         A0[i+36] = a0;
         a1 += w36 * z1;
         A1[i+36] = a1;
         a2 += w36 * z2;
         A2[i+36] = a2;
         a3 += w36 * z3;
         A3[i+36] = a3;
         a0 = A0[i+37];
         a0 += x37 * y0;
         a1 = A1[i+37];
         a1 += x37 * y1;
         a2 = A2[i+37];
         a2 += x37 * y2;
         a3 = A3[i+37];
         a3 += x37 * y3;
         a0 += w37 * z0;
         A0[i+37] = a0;
         a1 += w37 * z1;
         A1[i+37] = a1;
         a2 += w37 * z2;
         A2[i+37] = a2;
         a3 += w37 * z3;
         A3[i+37] = a3;
         a0 = A0[i+38];
         a0 += x38 * y0;
         a1 = A1[i+38];
         a1 += x38 * y1;
         a2 = A2[i+38];
         a2 += x38 * y2;
         a3 = A3[i+38];
         a3 += x38 * y3;
         a0 += w38 * z0;
         A0[i+38] = a0;
         a1 += w38 * z1;
         A1[i+38] = a1;
         a2 += w38 * z2;
         A2[i+38] = a2;
         a3 += w38 * z3;
         A3[i+38] = a3;
         a0 = A0[i+39];
         a0 += x39 * y0;
         a1 = A1[i+39];
         a1 += x39 * y1;
         a2 = A2[i+39];
         a2 += x39 * y2;
         a3 = A3[i+39];
         a3 += x39 * y3;
         a0 += w39 * z0;
         A0[i+39] = a0;
         a1 += w39 * z1;
         A1[i+39] = a1;
         a2 += w39 * z2;
         A2[i+39] = a2;
         a3 += w39 * z3;
         A3[i+39] = a3;
         a0 = A0[i+40];
         a0 += x40 * y0;
         a1 = A1[i+40];
         a1 += x40 * y1;
         a2 = A2[i+40];
         a2 += x40 * y2;
         a3 = A3[i+40];
         a3 += x40 * y3;
         a0 += w40 * z0;
         A0[i+40] = a0;
         a1 += w40 * z1;
         A1[i+40] = a1;
         a2 += w40 * z2;
         A2[i+40] = a2;
         a3 += w40 * z3;
         A3[i+40] = a3;
         a0 = A0[i+41];
         a0 += x41 * y0;
         a1 = A1[i+41];
         a1 += x41 * y1;
         a2 = A2[i+41];
         a2 += x41 * y2;
         a3 = A3[i+41];
         a3 += x41 * y3;
         a0 += w41 * z0;
         A0[i+41] = a0;
         a1 += w41 * z1;
         A1[i+41] = a1;
         a2 += w41 * z2;
         A2[i+41] = a2;
         a3 += w41 * z3;
         A3[i+41] = a3;
         a0 = A0[i+42];
         a0 += x42 * y0;
         a1 = A1[i+42];
         a1 += x42 * y1;
         a2 = A2[i+42];
         a2 += x42 * y2;
         a3 = A3[i+42];
         a3 += x42 * y3;
         a0 += w42 * z0;
         A0[i+42] = a0;
         a1 += w42 * z1;
         A1[i+42] = a1;
         a2 += w42 * z2;
         A2[i+42] = a2;
         a3 += w42 * z3;
         A3[i+42] = a3;
         a0 = A0[i+43];
         a0 += x43 * y0;
         a1 = A1[i+43];
         a1 += x43 * y1;
         a2 = A2[i+43];
         a2 += x43 * y2;
         a3 = A3[i+43];
         a3 += x43 * y3;
         a0 += w43 * z0;
         A0[i+43] = a0;
         a1 += w43 * z1;
         A1[i+43] = a1;
         a2 += w43 * z2;
         A2[i+43] = a2;
         a3 += w43 * z3;
         A3[i+43] = a3;
         a0 = A0[i+44];
         a0 += x44 * y0;
         a1 = A1[i+44];
         a1 += x44 * y1;
         a2 = A2[i+44];
         a2 += x44 * y2;
         a3 = A3[i+44];
         a3 += x44 * y3;
         a0 += w44 * z0;
         A0[i+44] = a0;
         a1 += w44 * z1;
         A1[i+44] = a1;
         a2 += w44 * z2;
         A2[i+44] = a2;
         a3 += w44 * z3;
         A3[i+44] = a3;
         a0 = A0[i+45];
         a0 += x45 * y0;
         a1 = A1[i+45];
         a1 += x45 * y1;
         a2 = A2[i+45];
         a2 += x45 * y2;
         a3 = A3[i+45];
         a3 += x45 * y3;
         a0 += w45 * z0;
         A0[i+45] = a0;
         a1 += w45 * z1;
         A1[i+45] = a1;
         a2 += w45 * z2;
         A2[i+45] = a2;
         a3 += w45 * z3;
         A3[i+45] = a3;
         a0 = A0[i+46];
         a0 += x46 * y0;
         a1 = A1[i+46];
         a1 += x46 * y1;
         a2 = A2[i+46];
         a2 += x46 * y2;
         a3 = A3[i+46];
         a3 += x46 * y3;
         a0 += w46 * z0;
         A0[i+46] = a0;
         a1 += w46 * z1;
         A1[i+46] = a1;
         a2 += w46 * z2;
         A2[i+46] = a2;
         a3 += w46 * z3;
         A3[i+46] = a3;
         a0 = A0[i+47];
         a0 += x47 * y0;
         a1 = A1[i+47];
         a1 += x47 * y1;
         a2 = A2[i+47];
         a2 += x47 * y2;
         a3 = A3[i+47];
         a3 += x47 * y3;
         a0 += w47 * z0;
         A0[i+47] = a0;
         a1 += w47 * z1;
         A1[i+47] = a1;
         a2 += w47 * z2;
         A2[i+47] = a2;
         a3 += w47 * z3;
         A3[i+47] = a3;
         a0 = A0[i+48];
         a0 += x48 * y0;
         a1 = A1[i+48];
         a1 += x48 * y1;
         a2 = A2[i+48];
         a2 += x48 * y2;
         a3 = A3[i+48];
         a3 += x48 * y3;
         a0 += w48 * z0;
         prefA(A0);
         A0[i+48] = a0;
         a1 += w48 * z1;
         prefA(A1);
         A1[i+48] = a1;
         a2 += w48 * z2;
         prefA(A2);
         A2[i+48] = a2;
         a3 += w48 * z3;
         prefA(A3);
         A3[i+48] = a3;
         a0 = A0[i+49];
         a0 += x49 * y0;
         a1 = A1[i+49];
         a1 += x49 * y1;
         a2 = A2[i+49];
         a2 += x49 * y2;
         a3 = A3[i+49];
         a3 += x49 * y3;
         a0 += w49 * z0;
         A0[i+49] = a0;
         a1 += w49 * z1;
         A1[i+49] = a1;
         a2 += w49 * z2;
         A2[i+49] = a2;
         a3 += w49 * z3;
         A3[i+49] = a3;
         a0 = A0[i+50];
         a0 += x50 * y0;
         a1 = A1[i+50];
         a1 += x50 * y1;
         a2 = A2[i+50];
         a2 += x50 * y2;
         a3 = A3[i+50];
         a3 += x50 * y3;
         a0 += w50 * z0;
         A0[i+50] = a0;
         a1 += w50 * z1;
         A1[i+50] = a1;
         a2 += w50 * z2;
         A2[i+50] = a2;
         a3 += w50 * z3;
         A3[i+50] = a3;
         a0 = A0[i+51];
         a0 += x51 * y0;
         a1 = A1[i+51];
         a1 += x51 * y1;
         a2 = A2[i+51];
         a2 += x51 * y2;
         a3 = A3[i+51];
         a3 += x51 * y3;
         a0 += w51 * z0;
         A0[i+51] = a0;
         a1 += w51 * z1;
         A1[i+51] = a1;
         a2 += w51 * z2;
         A2[i+51] = a2;
         a3 += w51 * z3;
         A3[i+51] = a3;
         a0 = A0[i+52];
         a0 += x52 * y0;
         a1 = A1[i+52];
         a1 += x52 * y1;
         a2 = A2[i+52];
         a2 += x52 * y2;
         a3 = A3[i+52];
         a3 += x52 * y3;
         a0 += w52 * z0;
         A0[i+52] = a0;
         a1 += w52 * z1;
         A1[i+52] = a1;
         a2 += w52 * z2;
         A2[i+52] = a2;
         a3 += w52 * z3;
         A3[i+52] = a3;
         a0 = A0[i+53];
         a0 += x53 * y0;
         a1 = A1[i+53];
         a1 += x53 * y1;
         a2 = A2[i+53];
         a2 += x53 * y2;
         a3 = A3[i+53];
         a3 += x53 * y3;
         a0 += w53 * z0;
         A0[i+53] = a0;
         a1 += w53 * z1;
         A1[i+53] = a1;
         a2 += w53 * z2;
         A2[i+53] = a2;
         a3 += w53 * z3;
         A3[i+53] = a3;
         a0 = A0[i+54];
         a0 += x54 * y0;
         a1 = A1[i+54];
         a1 += x54 * y1;
         a2 = A2[i+54];
         a2 += x54 * y2;
         a3 = A3[i+54];
         a3 += x54 * y3;
         a0 += w54 * z0;
         A0[i+54] = a0;
         a1 += w54 * z1;
         A1[i+54] = a1;
         a2 += w54 * z2;
         A2[i+54] = a2;
         a3 += w54 * z3;
         A3[i+54] = a3;
         a0 = A0[i+55];
         a0 += x55 * y0;
         a1 = A1[i+55];
         a1 += x55 * y1;
         a2 = A2[i+55];
         a2 += x55 * y2;
         a3 = A3[i+55];
         a3 += x55 * y3;
         a0 += w55 * z0;
         A0[i+55] = a0;
         a1 += w55 * z1;
         A1[i+55] = a1;
         a2 += w55 * z2;
         A2[i+55] = a2;
         a3 += w55 * z3;
         A3[i+55] = a3;
         a0 = A0[i+56];
         a0 += x56 * y0;
         a1 = A1[i+56];
         a1 += x56 * y1;
         a2 = A2[i+56];
         a2 += x56 * y2;
         a3 = A3[i+56];
         a3 += x56 * y3;
         a0 += w56 * z0;
         A0[i+56] = a0;
         a1 += w56 * z1;
         A1[i+56] = a1;
         a2 += w56 * z2;
         A2[i+56] = a2;
         a3 += w56 * z3;
         A3[i+56] = a3;
         a0 = A0[i+57];
         a0 += x57 * y0;
         a1 = A1[i+57];
         a1 += x57 * y1;
         a2 = A2[i+57];
         a2 += x57 * y2;
         a3 = A3[i+57];
         a3 += x57 * y3;
         a0 += w57 * z0;
         A0[i+57] = a0;
         a1 += w57 * z1;
         A1[i+57] = a1;
         a2 += w57 * z2;
         A2[i+57] = a2;
         a3 += w57 * z3;
         A3[i+57] = a3;
         a0 = A0[i+58];
         a0 += x58 * y0;
         a1 = A1[i+58];
         a1 += x58 * y1;
         a2 = A2[i+58];
         a2 += x58 * y2;
         a3 = A3[i+58];
         a3 += x58 * y3;
         a0 += w58 * z0;
         A0[i+58] = a0;
         a1 += w58 * z1;
         A1[i+58] = a1;
         a2 += w58 * z2;
         A2[i+58] = a2;
         a3 += w58 * z3;
         A3[i+58] = a3;
         a0 = A0[i+59];
         a0 += x59 * y0;
         a1 = A1[i+59];
         a1 += x59 * y1;
         a2 = A2[i+59];
         a2 += x59 * y2;
         a3 = A3[i+59];
         a3 += x59 * y3;
         a0 += w59 * z0;
         A0[i+59] = a0;
         a1 += w59 * z1;
         A1[i+59] = a1;
         a2 += w59 * z2;
         A2[i+59] = a2;
         a3 += w59 * z3;
         A3[i+59] = a3;
         a0 = A0[i+60];
         a0 += x60 * y0;
         a1 = A1[i+60];
         a1 += x60 * y1;
         a2 = A2[i+60];
         a2 += x60 * y2;
         a3 = A3[i+60];
         a3 += x60 * y3;
         a0 += w60 * z0;
         A0[i+60] = a0;
         a1 += w60 * z1;
         A1[i+60] = a1;
         a2 += w60 * z2;
         A2[i+60] = a2;
         a3 += w60 * z3;
         A3[i+60] = a3;
         a0 = A0[i+61];
         a0 += x61 * y0;
         a1 = A1[i+61];
         a1 += x61 * y1;
         a2 = A2[i+61];
         a2 += x61 * y2;
         a3 = A3[i+61];
         a3 += x61 * y3;
         a0 += w61 * z0;
         A0[i+61] = a0;
         a1 += w61 * z1;
         A1[i+61] = a1;
         a2 += w61 * z2;
         A2[i+61] = a2;
         a3 += w61 * z3;
         A3[i+61] = a3;
         a0 = A0[i+62];
         a0 += x62 * y0;
         a1 = A1[i+62];
         a1 += x62 * y1;
         a2 = A2[i+62];
         a2 += x62 * y2;
         a3 = A3[i+62];
         a3 += x62 * y3;
         a0 += w62 * z0;
         A0[i+62] = a0;
         a1 += w62 * z1;
         A1[i+62] = a1;
         a2 += w62 * z2;
         A2[i+62] = a2;
         a3 += w62 * z3;
         A3[i+62] = a3;
         a0 = A0[i+63];
         a0 += x63 * y0;
         a1 = A1[i+63];
         a1 += x63 * y1;
         a2 = A2[i+63];
         a2 += x63 * y2;
         a3 = A3[i+63];
         a3 += x63 * y3;
         a0 += w63 * z0;
         A0[i+63] = a0;
         a1 += w63 * z1;
         A1[i+63] = a1;
         a2 += w63 * z2;
         A2[i+63] = a2;
         a3 += w63 * z3;
         A3[i+63] = a3;
         a0 = A0[i+64];
         a0 += x64 * y0;
         a1 = A1[i+64];
         a1 += x64 * y1;
         a2 = A2[i+64];
         a2 += x64 * y2;
         a3 = A3[i+64];
         a3 += x64 * y3;
         a0 += w64 * z0;
         prefA(A0);
         A0[i+64] = a0;
         a1 += w64 * z1;
         prefA(A1);
         A1[i+64] = a1;
         a2 += w64 * z2;
         prefA(A2);
         A2[i+64] = a2;
         a3 += w64 * z3;
         prefA(A3);
         A3[i+64] = a3;
         a0 = A0[i+65];
         a0 += x65 * y0;
         a1 = A1[i+65];
         a1 += x65 * y1;
         a2 = A2[i+65];
         a2 += x65 * y2;
         a3 = A3[i+65];
         a3 += x65 * y3;
         a0 += w65 * z0;
         A0[i+65] = a0;
         a1 += w65 * z1;
         A1[i+65] = a1;
         a2 += w65 * z2;
         A2[i+65] = a2;
         a3 += w65 * z3;
         A3[i+65] = a3;
         a0 = A0[i+66];
         a0 += x66 * y0;
         a1 = A1[i+66];
         a1 += x66 * y1;
         a2 = A2[i+66];
         a2 += x66 * y2;
         a3 = A3[i+66];
         a3 += x66 * y3;
         a0 += w66 * z0;
         A0[i+66] = a0;
         a1 += w66 * z1;
         A1[i+66] = a1;
         a2 += w66 * z2;
         A2[i+66] = a2;
         a3 += w66 * z3;
         A3[i+66] = a3;
         a0 = A0[i+67];
         a0 += x67 * y0;
         a1 = A1[i+67];
         a1 += x67 * y1;
         a2 = A2[i+67];
         a2 += x67 * y2;
         a3 = A3[i+67];
         a3 += x67 * y3;
         a0 += w67 * z0;
         A0[i+67] = a0;
         a1 += w67 * z1;
         A1[i+67] = a1;
         a2 += w67 * z2;
         A2[i+67] = a2;
         a3 += w67 * z3;
         A3[i+67] = a3;
         a0 = A0[i+68];
         a0 += x68 * y0;
         a1 = A1[i+68];
         a1 += x68 * y1;
         a2 = A2[i+68];
         a2 += x68 * y2;
         a3 = A3[i+68];
         a3 += x68 * y3;
         a0 += w68 * z0;
         A0[i+68] = a0;
         a1 += w68 * z1;
         A1[i+68] = a1;
         a2 += w68 * z2;
         A2[i+68] = a2;
         a3 += w68 * z3;
         A3[i+68] = a3;
         a0 = A0[i+69];
         a0 += x69 * y0;
         a1 = A1[i+69];
         a1 += x69 * y1;
         a2 = A2[i+69];
         a2 += x69 * y2;
         a3 = A3[i+69];
         a3 += x69 * y3;
         a0 += w69 * z0;
         A0[i+69] = a0;
         a1 += w69 * z1;
         A1[i+69] = a1;
         a2 += w69 * z2;
         A2[i+69] = a2;
         a3 += w69 * z3;
         A3[i+69] = a3;
         a0 = A0[i+70];
         a0 += x70 * y0;
         a1 = A1[i+70];
         a1 += x70 * y1;
         a2 = A2[i+70];
         a2 += x70 * y2;
         a3 = A3[i+70];
         a3 += x70 * y3;
         a0 += w70 * z0;
         A0[i+70] = a0;
         a1 += w70 * z1;
         A1[i+70] = a1;
         a2 += w70 * z2;
         A2[i+70] = a2;
         a3 += w70 * z3;
         A3[i+70] = a3;
         a0 = A0[i+71];
         a0 += x71 * y0;
         a1 = A1[i+71];
         a1 += x71 * y1;
         a2 = A2[i+71];
         a2 += x71 * y2;
         a3 = A3[i+71];
         a3 += x71 * y3;
         a0 += w71 * z0;
         A0[i+71] = a0;
         a1 += w71 * z1;
         A1[i+71] = a1;
         a2 += w71 * z2;
         A2[i+71] = a2;
         a3 += w71 * z3;
         A3[i+71] = a3;
         a0 = A0[i+72];
         a0 += x72 * y0;
         a1 = A1[i+72];
         a1 += x72 * y1;
         a2 = A2[i+72];
         a2 += x72 * y2;
         a3 = A3[i+72];
         a3 += x72 * y3;
         a0 += w72 * z0;
         A0[i+72] = a0;
         a1 += w72 * z1;
         A1[i+72] = a1;
         a2 += w72 * z2;
         A2[i+72] = a2;
         a3 += w72 * z3;
         A3[i+72] = a3;
         a0 = A0[i+73];
         a0 += x73 * y0;
         a1 = A1[i+73];
         a1 += x73 * y1;
         a2 = A2[i+73];
         a2 += x73 * y2;
         a3 = A3[i+73];
         a3 += x73 * y3;
         a0 += w73 * z0;
         A0[i+73] = a0;
         a1 += w73 * z1;
         A1[i+73] = a1;
         a2 += w73 * z2;
         A2[i+73] = a2;
         a3 += w73 * z3;
         A3[i+73] = a3;
         a0 = A0[i+74];
         a0 += x74 * y0;
         a1 = A1[i+74];
         a1 += x74 * y1;
         a2 = A2[i+74];
         a2 += x74 * y2;
         a3 = A3[i+74];
         a3 += x74 * y3;
         a0 += w74 * z0;
         A0[i+74] = a0;
         a1 += w74 * z1;
         A1[i+74] = a1;
         a2 += w74 * z2;
         A2[i+74] = a2;
         a3 += w74 * z3;
         A3[i+74] = a3;
         a0 = A0[i+75];
         a0 += x75 * y0;
         a1 = A1[i+75];
         a1 += x75 * y1;
         a2 = A2[i+75];
         a2 += x75 * y2;
         a3 = A3[i+75];
         a3 += x75 * y3;
         a0 += w75 * z0;
         A0[i+75] = a0;
         a1 += w75 * z1;
         A1[i+75] = a1;
         a2 += w75 * z2;
         A2[i+75] = a2;
         a3 += w75 * z3;
         A3[i+75] = a3;
         a0 = A0[i+76];
         a0 += x76 * y0;
         a1 = A1[i+76];
         a1 += x76 * y1;
         a2 = A2[i+76];
         a2 += x76 * y2;
         a3 = A3[i+76];
         a3 += x76 * y3;
         a0 += w76 * z0;
         A0[i+76] = a0;
         a1 += w76 * z1;
         A1[i+76] = a1;
         a2 += w76 * z2;
         A2[i+76] = a2;
         a3 += w76 * z3;
         A3[i+76] = a3;
         a0 = A0[i+77];
         a0 += x77 * y0;
         a1 = A1[i+77];
         a1 += x77 * y1;
         a2 = A2[i+77];
         a2 += x77 * y2;
         a3 = A3[i+77];
         a3 += x77 * y3;
         a0 += w77 * z0;
         A0[i+77] = a0;
         a1 += w77 * z1;
         A1[i+77] = a1;
         a2 += w77 * z2;
         A2[i+77] = a2;
         a3 += w77 * z3;
         A3[i+77] = a3;
         a0 = A0[i+78];
         a0 += x78 * y0;
         a1 = A1[i+78];
         a1 += x78 * y1;
         a2 = A2[i+78];
         a2 += x78 * y2;
         a3 = A3[i+78];
         a3 += x78 * y3;
         a0 += w78 * z0;
         A0[i+78] = a0;
         a1 += w78 * z1;
         A1[i+78] = a1;
         a2 += w78 * z2;
         A2[i+78] = a2;
         a3 += w78 * z3;
         A3[i+78] = a3;
         a0 = A0[i+79];
         a0 += x79 * y0;
         a1 = A1[i+79];
         a1 += x79 * y1;
         a2 = A2[i+79];
         a2 += x79 * y2;
         a3 = A3[i+79];
         a3 += x79 * y3;
         a0 += w79 * z0;
         A0[i+79] = a0;
         a1 += w79 * z1;
         A1[i+79] = a1;
         a2 += w79 * z2;
         A2[i+79] = a2;
         a3 += w79 * z3;
         A3[i+79] = a3;
         a0 = A0[i+80];
         a0 += x80 * y0;
         a1 = A1[i+80];
         a1 += x80 * y1;
         a2 = A2[i+80];
         a2 += x80 * y2;
         a3 = A3[i+80];
         a3 += x80 * y3;
         a0 += w80 * z0;
         prefA(A0);
         A0[i+80] = a0;
         a1 += w80 * z1;
         prefA(A1);
         A1[i+80] = a1;
         a2 += w80 * z2;
         prefA(A2);
         A2[i+80] = a2;
         a3 += w80 * z3;
         prefA(A3);
         A3[i+80] = a3;
         a0 = A0[i+81];
         a0 += x81 * y0;
         a1 = A1[i+81];
         a1 += x81 * y1;
         a2 = A2[i+81];
         a2 += x81 * y2;
         a3 = A3[i+81];
         a3 += x81 * y3;
         a0 += w81 * z0;
         A0[i+81] = a0;
         a1 += w81 * z1;
         A1[i+81] = a1;
         a2 += w81 * z2;
         A2[i+81] = a2;
         a3 += w81 * z3;
         A3[i+81] = a3;
         a0 = A0[i+82];
         a0 += x82 * y0;
         a1 = A1[i+82];
         a1 += x82 * y1;
         a2 = A2[i+82];
         a2 += x82 * y2;
         a3 = A3[i+82];
         a3 += x82 * y3;
         a0 += w82 * z0;
         A0[i+82] = a0;
         a1 += w82 * z1;
         A1[i+82] = a1;
         a2 += w82 * z2;
         A2[i+82] = a2;
         a3 += w82 * z3;
         A3[i+82] = a3;
         a0 = A0[i+83];
         a0 += x83 * y0;
         a1 = A1[i+83];
         a1 += x83 * y1;
         a2 = A2[i+83];
         a2 += x83 * y2;
         a3 = A3[i+83];
         a3 += x83 * y3;
         a0 += w83 * z0;
         A0[i+83] = a0;
         a1 += w83 * z1;
         A1[i+83] = a1;
         a2 += w83 * z2;
         A2[i+83] = a2;
         a3 += w83 * z3;
         A3[i+83] = a3;
         a0 = A0[i+84];
         a0 += x84 * y0;
         a1 = A1[i+84];
         a1 += x84 * y1;
         a2 = A2[i+84];
         a2 += x84 * y2;
         a3 = A3[i+84];
         a3 += x84 * y3;
         a0 += w84 * z0;
         A0[i+84] = a0;
         a1 += w84 * z1;
         A1[i+84] = a1;
         a2 += w84 * z2;
         A2[i+84] = a2;
         a3 += w84 * z3;
         A3[i+84] = a3;
         a0 = A0[i+85];
         a0 += x85 * y0;
         a1 = A1[i+85];
         a1 += x85 * y1;
         a2 = A2[i+85];
         a2 += x85 * y2;
         a3 = A3[i+85];
         a3 += x85 * y3;
         a0 += w85 * z0;
         A0[i+85] = a0;
         a1 += w85 * z1;
         A1[i+85] = a1;
         a2 += w85 * z2;
         A2[i+85] = a2;
         a3 += w85 * z3;
         A3[i+85] = a3;
         a0 = A0[i+86];
         a0 += x86 * y0;
         a1 = A1[i+86];
         a1 += x86 * y1;
         a2 = A2[i+86];
         a2 += x86 * y2;
         a3 = A3[i+86];
         a3 += x86 * y3;
         a0 += w86 * z0;
         A0[i+86] = a0;
         a1 += w86 * z1;
         A1[i+86] = a1;
         a2 += w86 * z2;
         A2[i+86] = a2;
         a3 += w86 * z3;
         A3[i+86] = a3;
         a0 = A0[i+87];
         a0 += x87 * y0;
         a1 = A1[i+87];
         a1 += x87 * y1;
         a2 = A2[i+87];
         a2 += x87 * y2;
         a3 = A3[i+87];
         a3 += x87 * y3;
         a0 += w87 * z0;
         A0[i+87] = a0;
         a1 += w87 * z1;
         A1[i+87] = a1;
         a2 += w87 * z2;
         A2[i+87] = a2;
         a3 += w87 * z3;
         A3[i+87] = a3;
         a0 = A0[i+88];
         a0 += x88 * y0;
         a1 = A1[i+88];
         a1 += x88 * y1;
         a2 = A2[i+88];
         a2 += x88 * y2;
         a3 = A3[i+88];
         a3 += x88 * y3;
         a0 += w88 * z0;
         A0[i+88] = a0;
         a1 += w88 * z1;
         A1[i+88] = a1;
         a2 += w88 * z2;
         A2[i+88] = a2;
         a3 += w88 * z3;
         A3[i+88] = a3;
         a0 = A0[i+89];
         a0 += x89 * y0;
         a1 = A1[i+89];
         a1 += x89 * y1;
         a2 = A2[i+89];
         a2 += x89 * y2;
         a3 = A3[i+89];
         a3 += x89 * y3;
         a0 += w89 * z0;
         A0[i+89] = a0;
         a1 += w89 * z1;
         A1[i+89] = a1;
         a2 += w89 * z2;
         A2[i+89] = a2;
         a3 += w89 * z3;
         A3[i+89] = a3;
         a0 = A0[i+90];
         a0 += x90 * y0;
         a1 = A1[i+90];
         a1 += x90 * y1;
         a2 = A2[i+90];
         a2 += x90 * y2;
         a3 = A3[i+90];
         a3 += x90 * y3;
         a0 += w90 * z0;
         A0[i+90] = a0;
         a1 += w90 * z1;
         A1[i+90] = a1;
         a2 += w90 * z2;
         A2[i+90] = a2;
         a3 += w90 * z3;
         A3[i+90] = a3;
         a0 = A0[i+91];
         a0 += x91 * y0;
         a1 = A1[i+91];
         a1 += x91 * y1;
         a2 = A2[i+91];
         a2 += x91 * y2;
         a3 = A3[i+91];
         a3 += x91 * y3;
         a0 += w91 * z0;
         A0[i+91] = a0;
         a1 += w91 * z1;
         A1[i+91] = a1;
         a2 += w91 * z2;
         A2[i+91] = a2;
         a3 += w91 * z3;
         A3[i+91] = a3;
         a0 = A0[i+92];
         a0 += x92 * y0;
         a1 = A1[i+92];
         a1 += x92 * y1;
         a2 = A2[i+92];
         a2 += x92 * y2;
         a3 = A3[i+92];
         a3 += x92 * y3;
         a0 += w92 * z0;
         A0[i+92] = a0;
         a1 += w92 * z1;
         A1[i+92] = a1;
         a2 += w92 * z2;
         A2[i+92] = a2;
         a3 += w92 * z3;
         A3[i+92] = a3;
         a0 = A0[i+93];
         a0 += x93 * y0;
         a1 = A1[i+93];
         a1 += x93 * y1;
         a2 = A2[i+93];
         a2 += x93 * y2;
         a3 = A3[i+93];
         a3 += x93 * y3;
         a0 += w93 * z0;
         A0[i+93] = a0;
         a1 += w93 * z1;
         A1[i+93] = a1;
         a2 += w93 * z2;
         A2[i+93] = a2;
         a3 += w93 * z3;
         A3[i+93] = a3;
         a0 = A0[i+94];
         a0 += x94 * y0;
         a1 = A1[i+94];
         a1 += x94 * y1;
         a2 = A2[i+94];
         a2 += x94 * y2;
         a3 = A3[i+94];
         a3 += x94 * y3;
         a0 += w94 * z0;
         A0[i+94] = a0;
         a1 += w94 * z1;
         A1[i+94] = a1;
         a2 += w94 * z2;
         A2[i+94] = a2;
         a3 += w94 * z3;
         A3[i+94] = a3;
         a0 = A0[i+95];
         a0 += x95 * y0;
         a1 = A1[i+95];
         a1 += x95 * y1;
         a2 = A2[i+95];
         a2 += x95 * y2;
         a3 = A3[i+95];
         a3 += x95 * y3;
         a0 += w95 * z0;
         A0[i+95] = a0;
         a1 += w95 * z1;
         A1[i+95] = a1;
         a2 += w95 * z2;
         A2[i+95] = a2;
         a3 += w95 * z3;
         A3[i+95] = a3;
         a0 = A0[i+96];
         a0 += x96 * y0;
         a1 = A1[i+96];
         a1 += x96 * y1;
         a2 = A2[i+96];
         a2 += x96 * y2;
         a3 = A3[i+96];
         a3 += x96 * y3;
         a0 += w96 * z0;
         prefA(A0);
         A0[i+96] = a0;
         a1 += w96 * z1;
         prefA(A1);
         A1[i+96] = a1;
         a2 += w96 * z2;
         prefA(A2);
         A2[i+96] = a2;
         a3 += w96 * z3;
         prefA(A3);
         A3[i+96] = a3;
         a0 = A0[i+97];
         a0 += x97 * y0;
         a1 = A1[i+97];
         a1 += x97 * y1;
         a2 = A2[i+97];
         a2 += x97 * y2;
         a3 = A3[i+97];
         a3 += x97 * y3;
         a0 += w97 * z0;
         A0[i+97] = a0;
         a1 += w97 * z1;
         A1[i+97] = a1;
         a2 += w97 * z2;
         A2[i+97] = a2;
         a3 += w97 * z3;
         A3[i+97] = a3;
         a0 = A0[i+98];
         a0 += x98 * y0;
         a1 = A1[i+98];
         a1 += x98 * y1;
         a2 = A2[i+98];
         a2 += x98 * y2;
         a3 = A3[i+98];
         a3 += x98 * y3;
         a0 += w98 * z0;
         A0[i+98] = a0;
         a1 += w98 * z1;
         A1[i+98] = a1;
         a2 += w98 * z2;
         A2[i+98] = a2;
         a3 += w98 * z3;
         A3[i+98] = a3;
         a0 = A0[i+99];
         a0 += x99 * y0;
         a1 = A1[i+99];
         a1 += x99 * y1;
         a2 = A2[i+99];
         a2 += x99 * y2;
         a3 = A3[i+99];
         a3 += x99 * y3;
         a0 += w99 * z0;
         A0[i+99] = a0;
         a1 += w99 * z1;
         A1[i+99] = a1;
         a2 += w99 * z2;
         A2[i+99] = a2;
         a3 += w99 * z3;
         A3[i+99] = a3;
         a0 = A0[i+100];
         a0 += x100 * y0;
         a1 = A1[i+100];
         a1 += x100 * y1;
         a2 = A2[i+100];
         a2 += x100 * y2;
         a3 = A3[i+100];
         a3 += x100 * y3;
         a0 += w100 * z0;
         A0[i+100] = a0;
         a1 += w100 * z1;
         A1[i+100] = a1;
         a2 += w100 * z2;
         A2[i+100] = a2;
         a3 += w100 * z3;
         A3[i+100] = a3;
         a0 = A0[i+101];
         a0 += x101 * y0;
         a1 = A1[i+101];
         a1 += x101 * y1;
         a2 = A2[i+101];
         a2 += x101 * y2;
         a3 = A3[i+101];
         a3 += x101 * y3;
         a0 += w101 * z0;
         A0[i+101] = a0;
         a1 += w101 * z1;
         A1[i+101] = a1;
         a2 += w101 * z2;
         A2[i+101] = a2;
         a3 += w101 * z3;
         A3[i+101] = a3;
         a0 = A0[i+102];
         a0 += x102 * y0;
         a1 = A1[i+102];
         a1 += x102 * y1;
         a2 = A2[i+102];
         a2 += x102 * y2;
         a3 = A3[i+102];
         a3 += x102 * y3;
         a0 += w102 * z0;
         A0[i+102] = a0;
         a1 += w102 * z1;
         A1[i+102] = a1;
         a2 += w102 * z2;
         A2[i+102] = a2;
         a3 += w102 * z3;
         A3[i+102] = a3;
         a0 = A0[i+103];
         a0 += x103 * y0;
         a1 = A1[i+103];
         a1 += x103 * y1;
         a2 = A2[i+103];
         a2 += x103 * y2;
         a3 = A3[i+103];
         a3 += x103 * y3;
         a0 += w103 * z0;
         A0[i+103] = a0;
         a1 += w103 * z1;
         A1[i+103] = a1;
         a2 += w103 * z2;
         A2[i+103] = a2;
         a3 += w103 * z3;
         A3[i+103] = a3;
         a0 = A0[i+104];
         a0 += x104 * y0;
         a1 = A1[i+104];
         a1 += x104 * y1;
         a2 = A2[i+104];
         a2 += x104 * y2;
         a3 = A3[i+104];
         a3 += x104 * y3;
         a0 += w104 * z0;
         A0[i+104] = a0;
         a1 += w104 * z1;
         A1[i+104] = a1;
         a2 += w104 * z2;
         A2[i+104] = a2;
         a3 += w104 * z3;
         A3[i+104] = a3;
         a0 = A0[i+105];
         a0 += x105 * y0;
         a1 = A1[i+105];
         a1 += x105 * y1;
         a2 = A2[i+105];
         a2 += x105 * y2;
         a3 = A3[i+105];
         a3 += x105 * y3;
         a0 += w105 * z0;
         A0[i+105] = a0;
         a1 += w105 * z1;
         A1[i+105] = a1;
         a2 += w105 * z2;
         A2[i+105] = a2;
         a3 += w105 * z3;
         A3[i+105] = a3;
         a0 = A0[i+106];
         a0 += x106 * y0;
         a1 = A1[i+106];
         a1 += x106 * y1;
         a2 = A2[i+106];
         a2 += x106 * y2;
         a3 = A3[i+106];
         a3 += x106 * y3;
         a0 += w106 * z0;
         A0[i+106] = a0;
         a1 += w106 * z1;
         A1[i+106] = a1;
         a2 += w106 * z2;
         A2[i+106] = a2;
         a3 += w106 * z3;
         A3[i+106] = a3;
         a0 = A0[i+107];
         a0 += x107 * y0;
         a1 = A1[i+107];
         a1 += x107 * y1;
         a2 = A2[i+107];
         a2 += x107 * y2;
         a3 = A3[i+107];
         a3 += x107 * y3;
         a0 += w107 * z0;
         A0[i+107] = a0;
         a1 += w107 * z1;
         A1[i+107] = a1;
         a2 += w107 * z2;
         A2[i+107] = a2;
         a3 += w107 * z3;
         A3[i+107] = a3;
         a0 = A0[i+108];
         a0 += x108 * y0;
         a1 = A1[i+108];
         a1 += x108 * y1;
         a2 = A2[i+108];
         a2 += x108 * y2;
         a3 = A3[i+108];
         a3 += x108 * y3;
         a0 += w108 * z0;
         A0[i+108] = a0;
         a1 += w108 * z1;
         A1[i+108] = a1;
         a2 += w108 * z2;
         A2[i+108] = a2;
         a3 += w108 * z3;
         A3[i+108] = a3;
         a0 = A0[i+109];
         a0 += x109 * y0;
         a1 = A1[i+109];
         a1 += x109 * y1;
         a2 = A2[i+109];
         a2 += x109 * y2;
         a3 = A3[i+109];
         a3 += x109 * y3;
         a0 += w109 * z0;
         A0[i+109] = a0;
         a1 += w109 * z1;
         A1[i+109] = a1;
         a2 += w109 * z2;
         A2[i+109] = a2;
         a3 += w109 * z3;
         A3[i+109] = a3;
         a0 = A0[i+110];
         a0 += x110 * y0;
         a1 = A1[i+110];
         a1 += x110 * y1;
         a2 = A2[i+110];
         a2 += x110 * y2;
         a3 = A3[i+110];
         a3 += x110 * y3;
         a0 += w110 * z0;
         A0[i+110] = a0;
         a1 += w110 * z1;
         A1[i+110] = a1;
         a2 += w110 * z2;
         A2[i+110] = a2;
         a3 += w110 * z3;
         A3[i+110] = a3;
         a0 = A0[i+111];
         a0 += x111 * y0;
         a1 = A1[i+111];
         a1 += x111 * y1;
         a2 = A2[i+111];
         a2 += x111 * y2;
         a3 = A3[i+111];
         a3 += x111 * y3;
         a0 += w111 * z0;
         A0[i+111] = a0;
         a1 += w111 * z1;
         A1[i+111] = a1;
         a2 += w111 * z2;
         A2[i+111] = a2;
         a3 += w111 * z3;
         A3[i+111] = a3;

      }
      for (i=M112; i < M; i++)
      {
         const register float x0 = X[i], w0 = W[i];
         register float a0=A0[i], a1=A1[i], a2=A2[i], a3=A3[i];

         a0 += x0 * y0;
         a1 += x0 * y1;
         a2 += x0 * y2;
         a3 += x0 * y3;
         a0 += w0 * z0;
         A0[i] = a0;
         a1 += w0 * z1;
         A1[i] = a1;
         a2 += w0 * z2;
         A2[i] = a2;
         a3 += w0 * z3;
         A3[i] = a3;
      }
      Z += 4;
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N4; j; j--, A += lda, Y++)
   {
      float *A0=A;
      const register float y0=Y[0], z0=Z[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      prefY(Z+1+1-1);
      for (i=0; i < M112; i += 112)
      {
         register float a0;
         const float x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                     w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                     x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                     w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                     x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                     x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                     x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15],
                     x16=X[i+16], w16=W[i+16], x17=X[i+17], w17=W[i+17],
                     x18=X[i+18], w18=W[i+18], x19=X[i+19], w19=W[i+19],
                     x20=X[i+20], w20=W[i+20], x21=X[i+21], w21=W[i+21],
                     x22=X[i+22], w22=W[i+22], x23=X[i+23], w23=W[i+23],
                     x24=X[i+24], w24=W[i+24], x25=X[i+25], w25=W[i+25],
                     x26=X[i+26], w26=W[i+26], x27=X[i+27], w27=W[i+27],
                     x28=X[i+28], w28=W[i+28], x29=X[i+29], w29=W[i+29],
                     x30=X[i+30], w30=W[i+30], x31=X[i+31], w31=W[i+31],
                     x32=X[i+32], w32=W[i+32], x33=X[i+33], w33=W[i+33],
                     x34=X[i+34], w34=W[i+34], x35=X[i+35], w35=W[i+35],
                     x36=X[i+36], w36=W[i+36], x37=X[i+37], w37=W[i+37],
                     x38=X[i+38], w38=W[i+38], x39=X[i+39], w39=W[i+39],
                     x40=X[i+40], w40=W[i+40], x41=X[i+41], w41=W[i+41],
                     x42=X[i+42], w42=W[i+42], x43=X[i+43], w43=W[i+43],
                     x44=X[i+44], w44=W[i+44], x45=X[i+45], w45=W[i+45],
                     x46=X[i+46], w46=W[i+46], x47=X[i+47], w47=W[i+47],
                     x48=X[i+48], w48=W[i+48], x49=X[i+49], w49=W[i+49],
                     x50=X[i+50], w50=W[i+50], x51=X[i+51], w51=W[i+51],
                     x52=X[i+52], w52=W[i+52], x53=X[i+53], w53=W[i+53],
                     x54=X[i+54], w54=W[i+54], x55=X[i+55], w55=W[i+55],
                     x56=X[i+56], w56=W[i+56], x57=X[i+57], w57=W[i+57],
                     x58=X[i+58], w58=W[i+58], x59=X[i+59], w59=W[i+59],
                     x60=X[i+60], w60=W[i+60], x61=X[i+61], w61=W[i+61],
                     x62=X[i+62], w62=W[i+62], x63=X[i+63], w63=W[i+63],
                     x64=X[i+64], w64=W[i+64], x65=X[i+65], w65=W[i+65],
                     x66=X[i+66], w66=W[i+66], x67=X[i+67], w67=W[i+67],
                     x68=X[i+68], w68=W[i+68], x69=X[i+69], w69=W[i+69],
                     x70=X[i+70], w70=W[i+70], x71=X[i+71], w71=W[i+71],
                     x72=X[i+72], w72=W[i+72], x73=X[i+73], w73=W[i+73],
                     x74=X[i+74], w74=W[i+74], x75=X[i+75], w75=W[i+75],
                     x76=X[i+76], w76=W[i+76], x77=X[i+77], w77=W[i+77],
                     x78=X[i+78], w78=W[i+78], x79=X[i+79], w79=W[i+79],
                     x80=X[i+80], w80=W[i+80], x81=X[i+81], w81=W[i+81],
                     x82=X[i+82], w82=W[i+82], x83=X[i+83], w83=W[i+83],
                     x84=X[i+84], w84=W[i+84], x85=X[i+85], w85=W[i+85],
                     x86=X[i+86], w86=W[i+86], x87=X[i+87], w87=W[i+87],
                     x88=X[i+88], w88=W[i+88], x89=X[i+89], w89=W[i+89],
                     x90=X[i+90], w90=W[i+90], x91=X[i+91], w91=W[i+91],
                     x92=X[i+92], w92=W[i+92], x93=X[i+93], w93=W[i+93],
                     x94=X[i+94], w94=W[i+94], x95=X[i+95], w95=W[i+95],
                     x96=X[i+96], w96=W[i+96], x97=X[i+97], w97=W[i+97],
                     x98=X[i+98], w98=W[i+98], x99=X[i+99], w99=W[i+99],
                     x100=X[i+100], w100=W[i+100], x101=X[i+101], w101=W[i+101],
                     x102=X[i+102], w102=W[i+102], x103=X[i+103], w103=W[i+103],
                     x104=X[i+104], w104=W[i+104], x105=X[i+105], w105=W[i+105],
                     x106=X[i+106], w106=W[i+106], x107=X[i+107], w107=W[i+107],
                     x108=X[i+108], w108=W[i+108], x109=X[i+109], w109=W[i+109],
                     x110=X[i+110], w110=W[i+110], x111=X[i+111],
                     w111=W[i+111];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a0 += w8 * z0;
         A0[i+8] = a0;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a0 += w15 * z0;
         A0[i+15] = a0;
         a0 = A0[i+16];
         a0 += x16 * y0;
         a0 += w16 * z0;
         prefA(A0);
         A0[i+16] = a0;
         a0 = A0[i+17];
         a0 += x17 * y0;
         a0 += w17 * z0;
         A0[i+17] = a0;
         a0 = A0[i+18];
         a0 += x18 * y0;
         a0 += w18 * z0;
         A0[i+18] = a0;
         a0 = A0[i+19];
         a0 += x19 * y0;
         a0 += w19 * z0;
         A0[i+19] = a0;
         a0 = A0[i+20];
         a0 += x20 * y0;
         a0 += w20 * z0;
         A0[i+20] = a0;
         a0 = A0[i+21];
         a0 += x21 * y0;
         a0 += w21 * z0;
         A0[i+21] = a0;
         a0 = A0[i+22];
         a0 += x22 * y0;
         a0 += w22 * z0;
         A0[i+22] = a0;
         a0 = A0[i+23];
         a0 += x23 * y0;
         a0 += w23 * z0;
         A0[i+23] = a0;
         a0 = A0[i+24];
         a0 += x24 * y0;
         a0 += w24 * z0;
         A0[i+24] = a0;
         a0 = A0[i+25];
         a0 += x25 * y0;
         a0 += w25 * z0;
         A0[i+25] = a0;
         a0 = A0[i+26];
         a0 += x26 * y0;
         a0 += w26 * z0;
         A0[i+26] = a0;
         a0 = A0[i+27];
         a0 += x27 * y0;
         a0 += w27 * z0;
         A0[i+27] = a0;
         a0 = A0[i+28];
         a0 += x28 * y0;
         a0 += w28 * z0;
         A0[i+28] = a0;
         a0 = A0[i+29];
         a0 += x29 * y0;
         a0 += w29 * z0;
         A0[i+29] = a0;
         a0 = A0[i+30];
         a0 += x30 * y0;
         a0 += w30 * z0;
         A0[i+30] = a0;
         a0 = A0[i+31];
         a0 += x31 * y0;
         a0 += w31 * z0;
         A0[i+31] = a0;
         a0 = A0[i+32];
         a0 += x32 * y0;
         a0 += w32 * z0;
         prefA(A0);
         A0[i+32] = a0;
         a0 = A0[i+33];
         a0 += x33 * y0;
         a0 += w33 * z0;
         A0[i+33] = a0;
         a0 = A0[i+34];
         a0 += x34 * y0;
         a0 += w34 * z0;
         A0[i+34] = a0;
         a0 = A0[i+35];
         a0 += x35 * y0;
         a0 += w35 * z0;
         A0[i+35] = a0;
         a0 = A0[i+36];
         a0 += x36 * y0;
         a0 += w36 * z0;
         A0[i+36] = a0;
         a0 = A0[i+37];
         a0 += x37 * y0;
         a0 += w37 * z0;
         A0[i+37] = a0;
         a0 = A0[i+38];
         a0 += x38 * y0;
         a0 += w38 * z0;
         A0[i+38] = a0;
         a0 = A0[i+39];
         a0 += x39 * y0;
         a0 += w39 * z0;
         A0[i+39] = a0;
         a0 = A0[i+40];
         a0 += x40 * y0;
         a0 += w40 * z0;
         A0[i+40] = a0;
         a0 = A0[i+41];
         a0 += x41 * y0;
         a0 += w41 * z0;
         A0[i+41] = a0;
         a0 = A0[i+42];
         a0 += x42 * y0;
         a0 += w42 * z0;
         A0[i+42] = a0;
         a0 = A0[i+43];
         a0 += x43 * y0;
         a0 += w43 * z0;
         A0[i+43] = a0;
         a0 = A0[i+44];
         a0 += x44 * y0;
         a0 += w44 * z0;
         A0[i+44] = a0;
         a0 = A0[i+45];
         a0 += x45 * y0;
         a0 += w45 * z0;
         A0[i+45] = a0;
         a0 = A0[i+46];
         a0 += x46 * y0;
         a0 += w46 * z0;
         A0[i+46] = a0;
         a0 = A0[i+47];
         a0 += x47 * y0;
         a0 += w47 * z0;
         A0[i+47] = a0;
         a0 = A0[i+48];
         a0 += x48 * y0;
         a0 += w48 * z0;
         prefA(A0);
         A0[i+48] = a0;
         a0 = A0[i+49];
         a0 += x49 * y0;
         a0 += w49 * z0;
         A0[i+49] = a0;
         a0 = A0[i+50];
         a0 += x50 * y0;
         a0 += w50 * z0;
         A0[i+50] = a0;
         a0 = A0[i+51];
         a0 += x51 * y0;
         a0 += w51 * z0;
         A0[i+51] = a0;
         a0 = A0[i+52];
         a0 += x52 * y0;
         a0 += w52 * z0;
         A0[i+52] = a0;
         a0 = A0[i+53];
         a0 += x53 * y0;
         a0 += w53 * z0;
         A0[i+53] = a0;
         a0 = A0[i+54];
         a0 += x54 * y0;
         a0 += w54 * z0;
         A0[i+54] = a0;
         a0 = A0[i+55];
         a0 += x55 * y0;
         a0 += w55 * z0;
         A0[i+55] = a0;
         a0 = A0[i+56];
         a0 += x56 * y0;
         a0 += w56 * z0;
         A0[i+56] = a0;
         a0 = A0[i+57];
         a0 += x57 * y0;
         a0 += w57 * z0;
         A0[i+57] = a0;
         a0 = A0[i+58];
         a0 += x58 * y0;
         a0 += w58 * z0;
         A0[i+58] = a0;
         a0 = A0[i+59];
         a0 += x59 * y0;
         a0 += w59 * z0;
         A0[i+59] = a0;
         a0 = A0[i+60];
         a0 += x60 * y0;
         a0 += w60 * z0;
         A0[i+60] = a0;
         a0 = A0[i+61];
         a0 += x61 * y0;
         a0 += w61 * z0;
         A0[i+61] = a0;
         a0 = A0[i+62];
         a0 += x62 * y0;
         a0 += w62 * z0;
         A0[i+62] = a0;
         a0 = A0[i+63];
         a0 += x63 * y0;
         a0 += w63 * z0;
         A0[i+63] = a0;
         a0 = A0[i+64];
         a0 += x64 * y0;
         a0 += w64 * z0;
         prefA(A0);
         A0[i+64] = a0;
         a0 = A0[i+65];
         a0 += x65 * y0;
         a0 += w65 * z0;
         A0[i+65] = a0;
         a0 = A0[i+66];
         a0 += x66 * y0;
         a0 += w66 * z0;
         A0[i+66] = a0;
         a0 = A0[i+67];
         a0 += x67 * y0;
         a0 += w67 * z0;
         A0[i+67] = a0;
         a0 = A0[i+68];
         a0 += x68 * y0;
         a0 += w68 * z0;
         A0[i+68] = a0;
         a0 = A0[i+69];
         a0 += x69 * y0;
         a0 += w69 * z0;
         A0[i+69] = a0;
         a0 = A0[i+70];
         a0 += x70 * y0;
         a0 += w70 * z0;
         A0[i+70] = a0;
         a0 = A0[i+71];
         a0 += x71 * y0;
         a0 += w71 * z0;
         A0[i+71] = a0;
         a0 = A0[i+72];
         a0 += x72 * y0;
         a0 += w72 * z0;
         A0[i+72] = a0;
         a0 = A0[i+73];
         a0 += x73 * y0;
         a0 += w73 * z0;
         A0[i+73] = a0;
         a0 = A0[i+74];
         a0 += x74 * y0;
         a0 += w74 * z0;
         A0[i+74] = a0;
         a0 = A0[i+75];
         a0 += x75 * y0;
         a0 += w75 * z0;
         A0[i+75] = a0;
         a0 = A0[i+76];
         a0 += x76 * y0;
         a0 += w76 * z0;
         A0[i+76] = a0;
         a0 = A0[i+77];
         a0 += x77 * y0;
         a0 += w77 * z0;
         A0[i+77] = a0;
         a0 = A0[i+78];
         a0 += x78 * y0;
         a0 += w78 * z0;
         A0[i+78] = a0;
         a0 = A0[i+79];
         a0 += x79 * y0;
         a0 += w79 * z0;
         A0[i+79] = a0;
         a0 = A0[i+80];
         a0 += x80 * y0;
         a0 += w80 * z0;
         prefA(A0);
         A0[i+80] = a0;
         a0 = A0[i+81];
         a0 += x81 * y0;
         a0 += w81 * z0;
         A0[i+81] = a0;
         a0 = A0[i+82];
         a0 += x82 * y0;
         a0 += w82 * z0;
         A0[i+82] = a0;
         a0 = A0[i+83];
         a0 += x83 * y0;
         a0 += w83 * z0;
         A0[i+83] = a0;
         a0 = A0[i+84];
         a0 += x84 * y0;
         a0 += w84 * z0;
         A0[i+84] = a0;
         a0 = A0[i+85];
         a0 += x85 * y0;
         a0 += w85 * z0;
         A0[i+85] = a0;
         a0 = A0[i+86];
         a0 += x86 * y0;
         a0 += w86 * z0;
         A0[i+86] = a0;
         a0 = A0[i+87];
         a0 += x87 * y0;
         a0 += w87 * z0;
         A0[i+87] = a0;
         a0 = A0[i+88];
         a0 += x88 * y0;
         a0 += w88 * z0;
         A0[i+88] = a0;
         a0 = A0[i+89];
         a0 += x89 * y0;
         a0 += w89 * z0;
         A0[i+89] = a0;
         a0 = A0[i+90];
         a0 += x90 * y0;
         a0 += w90 * z0;
         A0[i+90] = a0;
         a0 = A0[i+91];
         a0 += x91 * y0;
         a0 += w91 * z0;
         A0[i+91] = a0;
         a0 = A0[i+92];
         a0 += x92 * y0;
         a0 += w92 * z0;
         A0[i+92] = a0;
         a0 = A0[i+93];
         a0 += x93 * y0;
         a0 += w93 * z0;
         A0[i+93] = a0;
         a0 = A0[i+94];
         a0 += x94 * y0;
         a0 += w94 * z0;
         A0[i+94] = a0;
         a0 = A0[i+95];
         a0 += x95 * y0;
         a0 += w95 * z0;
         A0[i+95] = a0;
         a0 = A0[i+96];
         a0 += x96 * y0;
         a0 += w96 * z0;
         prefA(A0);
         A0[i+96] = a0;
         a0 = A0[i+97];
         a0 += x97 * y0;
         a0 += w97 * z0;
         A0[i+97] = a0;
         a0 = A0[i+98];
         a0 += x98 * y0;
         a0 += w98 * z0;
         A0[i+98] = a0;
         a0 = A0[i+99];
         a0 += x99 * y0;
         a0 += w99 * z0;
         A0[i+99] = a0;
         a0 = A0[i+100];
         a0 += x100 * y0;
         a0 += w100 * z0;
         A0[i+100] = a0;
         a0 = A0[i+101];
         a0 += x101 * y0;
         a0 += w101 * z0;
         A0[i+101] = a0;
         a0 = A0[i+102];
         a0 += x102 * y0;
         a0 += w102 * z0;
         A0[i+102] = a0;
         a0 = A0[i+103];
         a0 += x103 * y0;
         a0 += w103 * z0;
         A0[i+103] = a0;
         a0 = A0[i+104];
         a0 += x104 * y0;
         a0 += w104 * z0;
         A0[i+104] = a0;
         a0 = A0[i+105];
         a0 += x105 * y0;
         a0 += w105 * z0;
         A0[i+105] = a0;
         a0 = A0[i+106];
         a0 += x106 * y0;
         a0 += w106 * z0;
         A0[i+106] = a0;
         a0 = A0[i+107];
         a0 += x107 * y0;
         a0 += w107 * z0;
         A0[i+107] = a0;
         a0 = A0[i+108];
         a0 += x108 * y0;
         a0 += w108 * z0;
         A0[i+108] = a0;
         a0 = A0[i+109];
         a0 += x109 * y0;
         a0 += w109 * z0;
         A0[i+109] = a0;
         a0 = A0[i+110];
         a0 += x110 * y0;
         a0 += w110 * z0;
         A0[i+110] = a0;
         a0 = A0[i+111];
         a0 += x111 * y0;
         a0 += w111 * z0;
         A0[i+111] = a0;

      }
      for (i=M112; i < M; i++)
      {
         const register float x0 = X[i], w0 = W[i];
         register float a0=A0[i];

         a0 += x0 * y0;
         a0 += w0 * z0;
         A0[i] = a0;
      }
      Z++;
   }
}
